import { ManagedErrorType } from "./managed-error";
export declare type ExecuteManagedSyncFunc = (managedSyncFuncExecArgs: any) => any;
export declare type ExecuteManagedAsyncFunc = (managedAsyncFuncExecArgs: any) => Promise<any>;
export interface ManagedFuncDef<TManagedFuncName extends string, TExecuteManageFunc> {
    name: TManagedFuncName;
    namespace?: string;
    errorTypes: Array<ManagedErrorType<string>>;
}
export declare type ExecuteManagedFuncOf<TManagedFuncDef extends ManagedFuncDef<any, any>> = TManagedFuncDef extends ManagedFuncDef<any, infer TExecuteManagedFunc> ? TExecuteManagedFunc : never;
export declare type ManagedSyncFuncDef<TManagedSyncFuncName extends string, TExecuteManagedSyncFunc extends ExecuteManagedSyncFunc> = ManagedFuncDef<TManagedSyncFuncName, TExecuteManagedSyncFunc>;
export declare type ManagedAsyncFuncDef<TManagedAsyncFuncName extends string, TExecuteManagedAsyncFunc extends ExecuteManagedAsyncFunc> = ManagedFuncDef<TManagedAsyncFuncName, TExecuteManagedAsyncFunc>;
export declare const defineManagedFunc: () => {
    name: <TManagedFuncName extends string>(managedFuncName: TManagedFuncName) => {
        namespace: (managedFuncNamespace?: string) => {
            errorTypes: (managedFuncErrorTypes: Array<ManagedErrorType<string>>) => {
                sync: () => {
                    executeType: <TExecuteManagedSyncFunc extends ExecuteManagedSyncFunc>() => { [ManagedSyncFuncName in TManagedFuncName as `${ManagedSyncFuncName}Def`]: ManagedSyncFuncDef<TManagedFuncName, TExecuteManagedSyncFunc>; };
                };
                async: () => {
                    executeType: <TExecuteManagedAsyncFunc extends ExecuteManagedAsyncFunc>() => { [ManagedAsyncFuncName in TManagedFuncName as `${ManagedAsyncFuncName}Def`]: ManagedAsyncFuncDef<TManagedFuncName, TExecuteManagedAsyncFunc>; };
                };
            };
        };
    };
};
export declare const getManagedFuncFullname: (managedFuncName: string, managedFuncNamespace?: string) => string;
export interface ManagedFuncConfig {
    isLoggingEnabled: boolean;
    isLoggingExecArgsEnabled: boolean;
    isLoggingExecResultEnabled: boolean;
}
export declare type ManagedSyncFunc<TManagedSyncFuncName extends string, TExecuteSyncManagedFunc extends ExecuteManagedSyncFunc> = (TExecuteSyncManagedFunc & {
    def: ManagedSyncFuncDef<TManagedSyncFuncName, TExecuteSyncManagedFunc>;
    config: ManagedFuncConfig;
});
export declare type ManagedAsyncFunc<TManagedAsyncFuncName extends string, TExecuteManagedAsyncFunc extends ExecuteManagedAsyncFunc> = (TExecuteManagedAsyncFunc & {
    def: ManagedAsyncFuncDef<TManagedAsyncFuncName, TExecuteManagedAsyncFunc>;
    config: ManagedFuncConfig;
});
export declare const declareManagedFunc: () => {
    name: <TManagedFuncName extends string>(managedFuncName: TManagedFuncName) => {
        namespace: (managedFuncNamespace?: string) => {
            errorTypes: (managedFuncErrorTypes: Array<ManagedErrorType<string>>) => {
                sync: () => {
                    execute: <TExecuteManagedSyncFunc extends ExecuteManagedSyncFunc>(executeManagedSyncFunc: TExecuteManagedSyncFunc) => { [ManagedSyncFuncName in TManagedFuncName]: ManagedSyncFunc<TManagedFuncName, TExecuteManagedSyncFunc>; };
                };
                async: () => {
                    execute: <TExecuteManagedAsyncFunc extends ExecuteManagedAsyncFunc>(executeManagedAsyncFunc: TExecuteManagedAsyncFunc) => { [ManagedAsyncFuncName in TManagedFuncName]: ManagedAsyncFunc<TManagedFuncName, TExecuteManagedAsyncFunc>; };
                };
            };
        };
    };
    sync: () => {
        def: <TManagedSyncFuncName extends string, TExecuteManagedSyncFunc_1 extends ExecuteManagedSyncFunc>(managedSyncFuncDef: ManagedSyncFuncDef<TManagedSyncFuncName, TExecuteManagedSyncFunc_1>) => {
            execute: (executeManagedSyncFunc: TExecuteManagedSyncFunc_1) => { [ManagedSyncFuncName_1 in TManagedSyncFuncName]: ManagedSyncFunc<TManagedSyncFuncName, TExecuteManagedSyncFunc_1>; };
        };
    };
    async: () => {
        def: <TManagedAsyncFuncName extends string, TExecuteManagedAsyncFunc_1 extends ExecuteManagedAsyncFunc>(managedAsyncFuncDef: ManagedAsyncFuncDef<TManagedAsyncFuncName, TExecuteManagedAsyncFunc_1>) => {
            execute: (executeManagedAsyncFunc: TExecuteManagedAsyncFunc_1) => { [ManagedAsyncFuncName_1 in TManagedAsyncFuncName]: ManagedAsyncFunc<TManagedAsyncFuncName, TExecuteManagedAsyncFunc_1>; };
        };
    };
};
//# sourceMappingURL=managed-func.d.ts.map